from .agent_conversation_history_settings import AgentConversationHistorySettings
from .agent_gatekeeper_settings import AgentGatekeeperSettings
from .agent_orchestration_settings import AgentOrchestrationSettings
from .agent_tool import AgentTool
from .agent_vectorization_settings import AgentVectorizationSettings
from .agent_workflows.agent_workflow_ai_model import AgentWorkflowAIModel
from .agent_workflows.external_agent_workflow import ExternalAgentWorkflow
from .agent_workflows.agent_workflow_base import AgentWorkflowBase
from .agent_workflows.azure_openai_assistants_agent_workflow import AzureOpenAIAssistantsAgentWorkflow
from .agent_workflows.langchain_expression_language_agent_workflow import LangChainExpressionLanguageAgentWorkflow
from .agent_workflows.langgraph_react_agent_workflow import LangGraphReactAgentWorkflow
from .agent_base import AgentBase
from .knowledge_management_agent import KnowledgeManagementAgent
from .knowledge_management_completion_request import KnowledgeManagementCompletionRequest
from .knowledge_management_index_configuration import KnowledgeManagementIndexConfiguration
from .resource_object_ids_model_base import ResourceObjectIdsModelBase



"""
Encapsulates properties an agent tool
"""
from typing import Optional, Dict
from pydantic import Field
from .resource_object_ids_model_base import ResourceObjectIdsModelBase

class AgentTool(ResourceObjectIdsModelBase):
    """
    Encapsulates properties for an agent tool.
    """
    name: str = Field(..., description="The name of the agent tool.")
    description: str = Field(..., description="The description of the agent tool.")
    package_name: str = Field(..., description="The package name of the agent tool. For internal tools, this value will be FoundationaLLM. For external tools, this value will be the name of the package.")
    properties: Optional[dict] = Field(default=[], description="A dictionary of properties for the agent tool.")



from pydantic import BaseModel, Field
from typing import Any, Self, Optional, Dict
from ..resource_object_ids_model_base import ResourceObjectIdsModelBase
from foundationallm.utils import ObjectUtils
from foundationallm.langchain.exceptions import LangChainException

class AgentWorkflowBase(ResourceObjectIdsModelBase):
    """
    The base class used for an agent workflow.
    """
    type: Optional[str] = Field(None, alias="type")
    workflow_host: str = Field(None, alias="workflow_host")
    name: str = Field(None, alias="name")
    package_name: str=Field(None, alias="package_name")
    properties: Optional[dict] = Field(default=[], description="A dictionary of properties for the agent workflow.")

    @staticmethod
    def from_object(obj: Any) -> Self:
        agent_workflow_base: AgentWorkflowBase = None
        try:
            agent_workflow_base = AgentWorkflowBase(**ObjectUtils.translate_keys(obj))
        except Exception as e:
            raise LangChainException(f"The Agent Workflow base model object provided is invalid. {str(e)}", 400)

        if agent_workflow_base is None:
            raise LangChainException("The Agent Workflow base model object provided is invalid.", 400)

        return agent_workflow_base



"""
Class: ToolFactory
Description: Factory class for creating tools based on the AgentTool configuration.
"""
from foundationallm.config import Configuration, UserIdentity
from foundationallm.langchain.common import FoundationaLLMToolBase
from foundationallm.langchain.exceptions import LangChainException
from foundationallm.langchain.tools import DALLEImageGenerationTool, FoundationaLLMContentSearchTool
from foundationallm.models.agents import AgentTool
from foundationallm.plugins import PluginManager, PluginManagerTypes

class ToolFactory:
    """
    Factory class for creating tools based on the AgentTool configuration.
    """
    FLLM_PACKAGE_NAME = "FoundationaLLM"
    DALLE_IMAGE_GENERATION_TOOL_NAME = "DALLEImageGeneration"
    FOUNDATIONALLM_CONTENT_SEARCH_TOOL = "FoundationaLLMContentSearchTool"

    def __init__(self, plugin_manager: PluginManager):
        """
        Initializes the tool factory.

        Parameters
        ----------
        plugin_manager : PluginManager
            The plugin manager object used to load external tools.
        """
        self.plugin_manager = plugin_manager

    def get_tool(
        self,
        agent_name: str,
        tool_config: AgentTool,
        objects: dict,
        user_identity: UserIdentity,
        config: Configuration
    ) -> FoundationaLLMToolBase:
        """
        Creates an instance of a tool based on the tool configuration.
        """

        # Use a cache key based on agent name, package name, and tool name to store the tool instance in the object cache.
        cache_key = f"{agent_name}|{tool_config.package_name}|{tool_config.name}"

        if cache_key in self.plugin_manager.object_cache:
            return self.plugin_manager.object_cache[cache_key]

        if tool_config.package_name == self.FLLM_PACKAGE_NAME:
            # internal tools
            match tool_config.name:
                case self.DALLE_IMAGE_GENERATION_TOOL_NAME:
                    tool = DALLEImageGenerationTool(tool_config, objects, user_identity, config)
                    self.plugin_manager.object_cache[cache_key] = tool
                case self.FOUNDATIONALLM_CONTENT_SEARCH_TOOL:
                    tool = FoundationaLLMContentSearchTool(tool_config, objects, user_identity, config)
            if tool is not None:
                return tool
        else:
            tool_plugin_manager = None

            if tool_config.package_name in self.plugin_manager.external_modules:
                tool_plugin_manager = next(( \
                    pm for pm \
                    in self.plugin_manager.external_modules[tool_config.package_name].plugin_managers \
                    if pm.plugin_manager_type == PluginManagerTypes.TOOLS), None)
                if tool_plugin_manager is None:
                    raise LangChainException(f"Tool plugin manager not found for package {tool_config.package_name}")
                tool = tool_plugin_manager.create_tool(tool_config, objects, user_identity, config)
                self.plugin_manager.object_cache[cache_key] = tool
                return tool
            else:
                raise LangChainException(f"Package {tool_config.package_name} not found in the list of external modules loaded by the package manager.")

        raise LangChainException(f"Tool {tool_config.name} not found in package {tool_config.package_name}")




from enum import Enum

class AuthenticationTypes(str, Enum):
    """Enumerator of the Authentication providers."""
    UNKNOWN = "Unknown"
    AZURE_IDENTITY = "AzureIdentity"
    API_KEY = "APIKey"
    CONNECTION_STRING = "ConnectionString"
    ACCOUNT_KEY = "AccountKey"



"""
Class: FoundationaLLMToolBase
Description: FoundationaLLM base class for tools that uses the AgentTool model for its configuration.
"""

from typing import List, Dict

# Platform imports
from azure.identity import DefaultAzureCredential
from logging import Logger
from opentelemetry.trace import Tracer

# LangChain imports
from langchain_core.tools import BaseTool

# FoundationaLLM imports
from foundationallm.config import Configuration, UserIdentity
from foundationallm.models.agents import AgentTool
from foundationallm.telemetry import Telemetry

class FoundationaLLMToolBase(BaseTool):
    """
    FoundationaLLM base class for tools that uses the AgentTool model for its configuration.
    """

    response_format: str = 'content_and_artifact'

    def __init__(self, tool_config: AgentTool, objects:Dict, user_identity:UserIdentity, config: Configuration):
        """ Initializes the FoundationaLLMToolBase class with the tool configuration. """
        super().__init__(
            name=tool_config.name,
            description=tool_config.description
        )
        self.tool_config = tool_config
        self.objects = objects
        self.user_identity = user_identity
        self.config = config

        self.logger: Logger = Telemetry.get_logger(self.name)
        self.tracer: Tracer = Telemetry.get_tracer(self.name)
        self.default_credential = DefaultAzureCredential(exclude_environment_credential=True)

    class Config:
        """ Pydantic configuration for FoundationaLLMToolBase. """
        extra = "allow"



from pydantic import Field
from typing import Any, Self, Optional
from foundationallm.models.resource_providers import ResourceBase
from foundationallm.utils import ObjectUtils
from foundationallm.langchain.exceptions import LangChainException

class AIModelBase(ResourceBase):
    """
    The base class used for AIModel resources.
    """
    endpoint_object_id: str = Field(description="The object ID of the APIEndpointConfiguration object providing the configuration for the API endpoint used to interact with the model.")
    version: Optional[str] = Field(description="The version of the AI model.")
    deployment_name: Optional[str] = Field(description="The deployment name for the AI model.")
    model_parameters: Optional[dict] = Field(default={}, description="A dictionary containing default values for model parameters.")

    @staticmethod
    def from_object(obj: Any) -> Self:

        ai_model: AIModelBase = None

        try:
            ai_model = AIModelBase(**ObjectUtils.translate_keys(obj))
        except Exception as e:
            raise LangChainException(f"The AI model object provided is invalid. {str(e)}", 400)
        
        if ai_model is None:
            raise LangChainException("The AI model object provided is invalid.", 400)

        return ai_model
